@@ -113,19 +113,19 @@ lock_build_name (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static char *
-lock_build_symlink_name (const char *fname)
+lock_build_symlink_name (const vfs_path_t * fname_vpath)
 {
-    char *fname_copy, *symlink_name;
-    char absolute_fname[PATH_MAX];
+    const char *elpath;
+    char *str_filename, *str_dirname, *symlink_name;
 
-    if (mc_realpath (fname, absolute_fname) == NULL)
-        return NULL;
+    /* get first path piece */
+    elpath = vfs_path_get_by_index (fname_vpath, 0)->path;
 
-    fname = x_basename (absolute_fname);
-    fname_copy = g_strdup (fname);
-    absolute_fname[fname - absolute_fname] = '\0';
-    symlink_name = g_strconcat (absolute_fname, ".#", fname_copy, (char *) NULL);
-    g_free (fname_copy);
+    str_filename = g_path_get_basename (elpath);
+    str_dirname = g_path_get_dirname (elpath);
+    symlink_name = g_strconcat (str_dirname, PATH_SEP_STR ".#", str_filename, (char *) NULL);
+    g_free (str_dirname);
+    g_free (str_filename);
 
     return symlink_name;
 }
@@ -193,45 +193,39 @@ lock_get_info (const char *lockfname)
    Warning: Might do screen refresh and lose edit->force */
 
 int
-lock_file (const char *fname)
+lock_file (const vfs_path_t * fname_vpath)
 {
-    char *lockfname, *newlock, *msg, *lock;
+    char *lockfname = NULL, *newlock, *msg, *lock;
     struct stat statbuf;
     struct lock_s *lockinfo;
-    gboolean symlink_ok;
-    vfs_path_t *vpath;
+    gboolean is_local;
+    gboolean symlink_ok = FALSE;
+    const char *elpath;
 
-    /* Just to be sure (and don't lock new file) */
-    if (fname == NULL || *fname == '\0')
+    if (fname_vpath == NULL)
         return 0;
 
-    fname = tilde_expand (fname);
-
-    vpath = vfs_path_from_str (fname);
+    elpath = vfs_path_get_by_index (fname_vpath, 0)->path;
+    /* Just to be sure (and don't lock new file) */
+    if (*elpath == '\0')
+        return 0;
 
     /* Locking on VFS is not supported */
-    if (!vfs_file_is_local (vpath))
+    is_local = vfs_file_is_local (fname_vpath);
+    if (is_local)
     {
-        g_free ((gpointer) fname);
-        vfs_path_free (vpath);
-        return 0;
+        /* Check if already locked */
+        lockfname = lock_build_symlink_name (fname_vpath);
     }
-    vfs_path_free (vpath);
 
-    /* Check if already locked */
-    lockfname = lock_build_symlink_name (fname);
-    g_free ((gpointer) fname);
-    if (lockfname == NULL)
+    if (!is_local || lockfname == NULL)
         return 0;
 
     if (lstat (lockfname, &statbuf) == 0)
     {
         lock = lock_get_info (lockfname);
         if (lock == NULL)
-        {
-            g_free (lockfname);
-            return 0;
-        }
+            goto ret;
         lockinfo = lock_extract_info (lock);
 
         /* Check if locking process alive, ask user if required */
@@ -250,9 +244,9 @@ lock_file (const char *fname)
                 break;
             case 1:
             case -1:
-                g_free (lockfname);
                 g_free (msg);
-                return 0;
+                goto ret;
+                break;          /* FIXME: unneeded? */
             }
             g_free (msg);
         }
@@ -263,8 +257,9 @@ lock_file (const char *fname)
     newlock = lock_build_name ();
     symlink_ok = (symlink (newlock, lockfname) != -1);
     g_free (newlock);
-    g_free (lockfname);
 
+  ret:
+    g_free (lockfname);
     return symlink_ok ? 1 : 0;
 }
 
@@ -275,42 +270,41 @@ lock_file (const char *fname)
  */
 
 int
-unlock_file (const char *fname)
+unlock_file (const vfs_path_t * fname_vpath)
 {
     char *lockfname, *lock;
     struct stat statbuf;
+    const char *elpath;
 
-    /* Just to be sure */
-    if (fname == NULL || *fname == '\0')
+    if (fname_vpath == NULL)
         return 0;
 
-    fname = tilde_expand (fname);
-    lockfname = lock_build_symlink_name (fname);
-    g_free ((gpointer) fname);
+    elpath = vfs_path_get_by_index (fname_vpath, 0)->path;
+    /* Just to be sure (and don't lock new file) */
+    if (*elpath == '\0')
+        return 0;
+
+    lockfname = lock_build_symlink_name (fname_vpath);
 
     if (lockfname == NULL)
         return 0;
 
     /* Check if lock exists */
     if (lstat (lockfname, &statbuf) == -1)
-    {
-        g_free (lockfname);
-        return 0;
-    }
+        goto ret;
 
     lock = lock_get_info (lockfname);
     if (lock != NULL)
     {
         /* Don't touch if lock is not ours */
         if (lock_extract_info (lock)->pid != getpid ())
-        {
-            g_free (lockfname);
-            return 0;
-        }
+            goto ret;
     }
 
     /* Remove lock */
     unlink (lockfname);
+
+  ret:
     g_free (lockfname);
     return 0;
 }
