@@ -2,27 +2,26 @@
    Search text engine.
    HEX-style pattern matching
 
-   Copyright (C) 2009 The Free Software Foundation, Inc.
+   Copyright (C) 2009, 2011
+   The Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2009.
 
    This file is part of the Midnight Commander.
 
-   The Midnight Commander is free software; you can redistribute it
+   The Midnight Commander is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
 
-   The Midnight Commander is distributed in the hope that it will be
-   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   The Midnight Commander is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -52,12 +51,25 @@ static GString *
 mc_search__hex_translate_to_regex (const GString * astr)
 {
     GString *buff;
-    gchar *tmp_str;
+    gchar *tmp_str, *tmp_str2;
     gsize tmp_str_len;
     gsize loop = 0;
 
     buff = g_string_sized_new (64);
     tmp_str = g_strndup (astr->str, astr->len);
+    tmp_str2 = tmp_str;
+
+    /* remove 0x prefices */
+    while (TRUE)
+    {
+        tmp_str2 = strstr (tmp_str2, "0x");
+        if (tmp_str2 == NULL)
+            break;
+
+        *tmp_str2++ = ' ';
+        *tmp_str2++ = ' ';
+    }
+
     g_strchug (tmp_str);        /* trim leadind whitespaces */
     tmp_str_len = strlen (tmp_str);
 
@@ -65,7 +77,7 @@ mc_search__hex_translate_to_regex (const GString * astr)
     {
         int val, ptr;
 
-        if (sscanf (tmp_str + loop, "%i%n", &val, &ptr))
+        if (sscanf (tmp_str + loop, "%x%n", &val, &ptr))
         {
             if (val < -128 || val > 255)
                 loop++;
@@ -108,6 +120,7 @@ mc_search__cond_struct_new_init_hex (const char *charset, mc_search_t * lc_mc_se
 {
     GString *tmp;
 
+    g_string_ascii_down (mc_search_cond->str);
     tmp = mc_search__hex_translate_to_regex (mc_search_cond->str);
     g_string_free (mc_search_cond->str, TRUE);
     mc_search_cond->str = tmp;
