@@ -509,54 +509,68 @@ tty_getyx (int *py, int *px)
 }
 
 /* --------------------------------------------------------------------------------------------- */
-/* if x < 0 or y < 0, draw line staring from current position */
 
 void
 tty_draw_hline (int y, int x, int ch, int len)
 {
+    int x1;
+
+    if (y < 0 || y >= LINES || x >= COLS)
+        return;
+
+    x1 = x;
+
+    if (x < 0)
+    {
+        len += x;
+        if (len <= 0)
+            return;
+        x = 0;
+    }
+
     if (ch == ACS_HLINE)
         ch = mc_tty_frm[MC_TTY_FRM_HORIZ];
-
-    if ((y < 0) || (x < 0))
-    {
-        y = SLsmg_get_row ();
-        x = SLsmg_get_column ();
-    }
-    else
-        SLsmg_gotorc (y, x);
-
     if (ch == 0)
         ch = ACS_HLINE;
 
+    SLsmg_gotorc (y, x);
+
     if (ch == ACS_HLINE)
         SLsmg_draw_hline (len);
     else
         while (len-- != 0)
             tty_print_char (ch);
 
-    SLsmg_gotorc (y, x);
+    SLsmg_gotorc (y, x1);
 }
 
 /* --------------------------------------------------------------------------------------------- */
-/* if x < 0 or y < 0, draw line staring from current position */
 
 void
 tty_draw_vline (int y, int x, int ch, int len)
 {
+    int y1;
+
+    if (x < 0 || x >= COLS || y >= LINES)
+        return;
+
+    y1 = y;
+
+    if (y < 0)
+    {
+        len += y;
+        if (len <= 0)
+            return;
+        y = 0;
+    }
+
     if (ch == ACS_VLINE)
         ch = mc_tty_frm[MC_TTY_FRM_VERT];
-
-    if ((y < 0) || (x < 0))
-    {
-        y = SLsmg_get_row ();
-        x = SLsmg_get_column ();
-    }
-    else
-        SLsmg_gotorc (y, x);
-
     if (ch == 0)
         ch = ACS_VLINE;
 
+    SLsmg_gotorc (y, x);
+
     if (ch == ACS_VLINE)
         SLsmg_draw_vline (len);
     else
@@ -571,7 +585,7 @@ tty_draw_vline (int y, int x, int ch, int len)
         }
     }
 
-    SLsmg_gotorc (y, x);
+    SLsmg_gotorc (y1, x);
 }
 
 /* --------------------------------------------------------------------------------------------- */
