@@ -177,20 +177,28 @@ tty_print_one_vline (gboolean single)
 void
 tty_draw_box (int y, int x, int ys, int xs, gboolean single)
 {
+    int y2, x2;
+
+    if (ys <= 0 || xs <= 0)
+        return;
+
     ys--;
     xs--;
 
+    y2 = y + ys;
+    x2 = x + xs;
+
     tty_draw_vline (y, x, mc_tty_frm[single ? MC_TTY_FRM_VERT : MC_TTY_FRM_DVERT], ys);
-    tty_draw_vline (y, x + xs, mc_tty_frm[single ? MC_TTY_FRM_VERT : MC_TTY_FRM_DVERT], ys);
+    tty_draw_vline (y, x2, mc_tty_frm[single ? MC_TTY_FRM_VERT : MC_TTY_FRM_DVERT], ys);
     tty_draw_hline (y, x, mc_tty_frm[single ? MC_TTY_FRM_HORIZ : MC_TTY_FRM_DHORIZ], xs);
-    tty_draw_hline (y + ys, x, mc_tty_frm[single ? MC_TTY_FRM_HORIZ : MC_TTY_FRM_DHORIZ], xs);
+    tty_draw_hline (y2, x, mc_tty_frm[single ? MC_TTY_FRM_HORIZ : MC_TTY_FRM_DHORIZ], xs);
     tty_gotoyx (y, x);
     tty_print_alt_char (ACS_ULCORNER, single);
-    tty_gotoyx (y + ys, x);
+    tty_gotoyx (y2, x);
     tty_print_alt_char (ACS_LLCORNER, single);
-    tty_gotoyx (y, x + xs);
+    tty_gotoyx (y, x2);
     tty_print_alt_char (ACS_URCORNER, single);
-    tty_gotoyx (y + ys, x + xs);
+    tty_gotoyx (y2, x2);
     tty_print_alt_char (ACS_LRCORNER, single);
 }
 
