@@ -15,7 +15,6 @@ extern gboolean mc_args__force_xterm;
 extern gboolean mc_args__nomouse;
 extern gboolean mc_args__force_colors;
 extern gboolean mc_args__nokeymap;
-extern gboolean mc_args__version;
 extern int mc_args__edit_start_line;
 extern char *mc_args__last_wd_file;
 extern char *mc_args__netfs_logfile;
@@ -24,7 +23,9 @@ extern int mc_args__debug_level;
 
 /*** declarations of public functions ************************************************************/
 
-gboolean mc_args_handle (int argc, char **argv, const char *translation_domain);
+gboolean mc_args_parse (int *argc, char ***argv, const char *translation_domain, GError ** error);
+gboolean mc_args_show_info (void);
+gboolean mc_setup_by_args (int argc, char **argv, GError ** error);
 
 /*** inline functions ****************************************************************************/
 #endif /* MC__ARGS_H */
