@@ -59,7 +59,9 @@
 #include "lib/widget.h"
 
 #include "src/setup.h"          /* For profile_name */
+#ifdef ENABLE_BACKGROUND
 #include "src/background.h"     /* task_list */
+#endif
 
 #ifdef HAVE_CHARSET
 #include "lib/charsets.h"
@@ -88,12 +90,12 @@
 #endif /* ENABLE_VFS_FTP */
 #endif /* ENABLE_VFS */
 
-#ifdef WITH_BACKGROUND
+#ifdef ENABLE_BACKGROUND
 #define B_STOP   (B_USER+1)
 #define B_RESUME (B_USER+2)
 #define B_KILL   (B_USER+3)
 #define JOBS_Y 15
-#endif /* WITH_BACKGROUND */
+#endif /* ENABLE_BACKGROUND */
 
 /*** file scope type declarations ****************************************************************/
 
@@ -121,7 +123,7 @@ static char *ret_directory_timeout;
 #endif /* ENABLE_VFS_FTP */
 #endif /* ENABLE_VFS */
 
-#ifdef WITH_BACKGROUND
+#ifdef ENABLE_BACKGROUND
 static int JOBS_X = 60;
 static WListbox *bg_list;
 static Dlg_head *jobs_dlg;
@@ -145,7 +147,7 @@ job_buttons[] =
     /* *INDENT-ON* */
 };
 
-#endif /* WITH_BACKGROUND */
+#endif /* ENABLE_BACKGROUND */
 
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -449,7 +451,7 @@ confvfs_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void *
 #endif /* ENABLE_VFS_FTP */
 #endif /* ENABLE_VFS */
 
-#ifdef WITH_BACKGROUND
+#ifdef ENABLE_BACKGROUND
 static void
 jobs_fill_listbox (void)
 {
@@ -519,7 +521,7 @@ task_cb (WButton * button, int action)
 
     return 0;
 }
-#endif /* WITH_BACKGROUND */
+#endif /* ENABLE_BACKGROUND */
 
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
@@ -916,7 +918,7 @@ tree_box (const char *current_dir)
     ((Widget *) bar)->y = LINES - 1;
 
     if (run_dlg (dlg) == B_ENTER)
-        val = g_strdup (tree_selected_name (mytree));
+        val = vfs_path_to_str (tree_selected_name (mytree));
 
     destroy_dlg (dlg);
     return val;
@@ -1041,8 +1043,12 @@ cd_dialog (void)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-symlink_dialog (const char *existing, const char *new, char **ret_existing, char **ret_new)
+symlink_dialog (const vfs_path_t * existing_vpath, const vfs_path_t * new_vpath,
+                char **ret_existing, char **ret_new)
 {
+    char *existing = vfs_path_to_str (existing_vpath);
+    char *new = vfs_path_to_str (new_vpath);
+
     QuickWidget quick_widgets[] = {
         /* 0 */ QUICK_BUTTON (50, 80, 6, 8, N_("&Cancel"), B_CANCEL, NULL),
         /* 1 */ QUICK_BUTTON (16, 80, 6, 8, N_("&OK"), B_ENTER, NULL),
@@ -1064,11 +1070,13 @@ symlink_dialog (const char *existing, const char *new, char **ret_existing, char
         *ret_new = NULL;
         *ret_existing = NULL;
     }
+    g_free (existing);
+    g_free (new);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-#ifdef WITH_BACKGROUND
+#ifdef ENABLE_BACKGROUND
 void
 jobs_cmd (void)
 {
@@ -1122,7 +1130,7 @@ jobs_cmd (void)
 
     destroy_dlg (jobs_dlg);
 }
-#endif /* WITH_BACKGROUND */
+#endif /* ENABLE_BACKGROUND */
 
 /* --------------------------------------------------------------------------------------------- */
 
