@@ -19,13 +19,14 @@
 /*** declarations of public functions ************************************************************/
 
 int display_box (WPanel * p, char **user, char **mini, int *use_msformat, int num);
-const panel_field_t *sort_box (panel_sort_info_t *info);
+const panel_field_t *sort_box (panel_sort_info_t * info);
 void confirm_box (void);
 void display_bits_box (void);
 void configure_vfs (void);
 void jobs_cmd (void);
 char *cd_dialog (void);
-void symlink_dialog (const char *existing, const char *new, char **ret_existing, char **ret_new);
+void symlink_dialog (const vfs_path_t * existing_vpath, const vfs_path_t * new_vpath,
+                     char **ret_existing, char **ret_new);
 char *tree_box (const char *current_dir);
 
 /*** inline functions ****************************************************************************/
