@@ -14,7 +14,7 @@
 
 /*** declarations of public functions ************************************************************/
 
-int regex_command (const char *filename, const char *action, int *move_dir);
+int regex_command (const vfs_path_t * filename_vpath, const char *action, int *move_dir);
 
 /* Call it after the user has edited the mc.ext file,
  * to flush the cached mc.ext file
