@@ -119,7 +119,14 @@ info_show_info (struct WInfo *info)
     if (get_current_type () != view_listing)
         return;
 
-    my_statfs (&myfs_stats, current_panel->cwd);
+    {
+        char *cwd_str;
+
+        cwd_str = vfs_path_to_str (current_panel->cwd_vpath);
+        my_statfs (&myfs_stats, cwd_str);
+        g_free (cwd_str);
+    }
+
     st = current_panel->dir.list[current_panel->selected].st;
 
     /* Print only lines which fit */
@@ -314,14 +321,17 @@ info_callback (Widget * w, widget_msg_t msg, int parm)
 static int
 info_event (Gpm_Event * event, void *data)
 {
-    Widget *w = &((WInfo *) data)->widget;
+    Widget *w = (Widget *) data;
+    Gpm_Event local;
+
+    if (!mouse_global_in_widget (event, w))
+        return MOU_UNHANDLED;
+
+    local = mouse_get_local (event, w);
 
     /* rest of the upper frame, the menu is invisible - call menu */
-    if (event->type & GPM_DOWN && event->y == 1 && !menubar_visible)
-    {
-        event->x += w->x;
+    if ((local.type & GPM_DOWN) != 0 && local.y == 1 && !menubar_visible)
         return the_menubar->widget.mouse (event, the_menubar);
-    }
 
     return MOU_NORMAL;
 }
