@@ -151,9 +151,13 @@ treebox_cmd (void)
     char *sel_dir;
 
     sel_dir = tree_box (selection (current_panel)->fname);
-    if (sel_dir)
+    if (sel_dir != NULL)
     {
-        do_cd (sel_dir, cd_exact);
+        vfs_path_t *sel_vdir;
+
+        sel_vdir = vfs_path_from_str (sel_dir);
+        do_cd (sel_vdir, cd_exact);
+        vfs_path_free (sel_vdir);
         g_free (sel_dir);
     }
 }
@@ -284,7 +288,7 @@ create_command_menu (void)
 #ifdef ENABLE_VFS
     entries = g_list_prepend (entries, menu_entry_create (_("&Active VFS list"), CK_VfsList));
 #endif
-#ifdef WITH_BACKGROUND
+#ifdef ENABLE_BACKGROUND
     entries = g_list_prepend (entries, menu_entry_create (_("&Background jobs"), CK_Jobs));
 #endif
     entries = g_list_prepend (entries, menu_entry_create (_("Screen lis&t"), CK_ScreenList));
@@ -425,36 +429,21 @@ midnight_get_shortcut (unsigned long command)
 static char *
 midnight_get_title (const Dlg_head * h, size_t len)
 {
-    /* TODO: share code with update_xterm_title_path() */
-
-    const char *path;
-    char host[BUF_TINY];
+    char *path;
+    char *login;
     char *p;
-    struct passwd *pw = NULL;
-    char *login = NULL;
-    int res = 0;
 
     (void) h;
 
-    path = strip_home_and_password (current_panel->cwd);
-    res = gethostname (host, sizeof (host));
-    if (res != 0)
-        host[0] = '\0';
-    else
-        host[sizeof (host) - 1] = '\0';
-
-    pw = getpwuid (getuid ());
-    if (pw != NULL)
-        login = g_strdup_printf ("%s@%s", pw->pw_name, host);
-    else
-        login = g_strdup (host);
+    title_path_prepare (&path, &login);
 
     p = g_strdup_printf ("%s [%s]:%s", _("Panels:"), login, path);
-    path = str_trunc (p, len - 4);
+    g_free (path);
     g_free (login);
+    path = g_strdup (str_trunc (p, len - 4));
     g_free (p);
 
-    return g_strdup (path);
+    return path;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -462,15 +451,14 @@ midnight_get_title (const Dlg_head * h, size_t len)
 static void
 toggle_panels_split (void)
 {
-    horizontal_split = !horizontal_split;
+    panels_layout.horizontal_split = !panels_layout.horizontal_split;
     layout_change ();
     do_refresh ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-#if ENABLE_VFS
-
+#ifdef ENABLE_VFS
 /* event helper */
 static gboolean
 check_panel_timestamp (const WPanel * panel, panel_view_mode_t mode, struct vfs_class *vclass,
@@ -478,24 +466,15 @@ check_panel_timestamp (const WPanel * panel, panel_view_mode_t mode, struct vfs_
 {
     if (mode == view_listing)
     {
-        vfs_path_t *vpath;
-        vfs_path_element_t *path_element;
+        const vfs_path_element_t *path_element;
 
-        vpath = vfs_path_from_str (panel->cwd);
-        path_element = vfs_path_get_by_index (vpath, -1);
+        path_element = vfs_path_get_by_index (panel->cwd_vpath, -1);
 
         if (path_element->class != vclass)
-        {
-            vfs_path_free (vpath);
             return FALSE;
-        }
 
-        if (vfs_getid (vpath) != id)
-        {
-            vfs_path_free (vpath);
+        if (vfs_getid (panel->cwd_vpath) != id)
             return FALSE;
-        }
-        vfs_path_free (vpath);
     }
     return TRUE;
 }
@@ -554,7 +533,7 @@ print_vfs_message (const gchar * event_group_name, const gchar * event_name,
 
     g_vsnprintf (str, sizeof (str), event_data->msg, event_data->ap);
 
-    if (mc_global.widget.midnight_shutdown)
+    if (mc_global.midnight_shutdown)
         return TRUE;
 
     if (!mc_global.message_visible || !the_hint || !the_hint->widget.owner)
@@ -591,7 +570,7 @@ create_panels (void)
     int current_index;
     int other_index;
     panel_view_mode_t current_mode, other_mode;
-    char original_dir[BUF_1K] = "\0";
+    vfs_path_t *original_dir = NULL;
 
     if (boot_current_is_left)
     {
@@ -610,25 +589,36 @@ create_panels (void)
     /* Creates the left panel */
     if (mc_run_param0 != NULL)
     {
+        vfs_path_t *vpath;
+
         if (mc_run_param1 != NULL)
         {
             /* Ok, user has specified two dirs, save the original one,
              * since we may not be able to chdir to the proper
              * second directory later
              */
-            mc_get_current_wd (original_dir, sizeof (original_dir) - 2);
+            original_dir = vfs_path_clone (vfs_get_raw_current_dir ());
         }
-        mc_chdir (mc_run_param0);
+        vpath = vfs_path_from_str (mc_run_param0);
+        mc_chdir (vpath);
+        vfs_path_free (vpath);
     }
     set_display_type (current_index, current_mode);
 
     /* The other panel */
     if (mc_run_param1 != NULL)
     {
-        if (original_dir[0] != '\0')
+        vfs_path_t *vpath;
+
+        if (original_dir != NULL)
             mc_chdir (original_dir);
-        mc_chdir (mc_run_param1);
+
+        vpath = vfs_path_from_str (mc_run_param1);
+        mc_chdir (vpath);
+        vfs_path_free (vpath);
     }
+    vfs_path_free (original_dir);
+
     set_display_type (other_index, other_mode);
 
     if (startup_left_mode == view_listing)
@@ -638,7 +628,7 @@ create_panels (void)
     else
         current_panel = left_panel;
 
-#if ENABLE_VFS
+#ifdef ENABLE_VFS
     mc_event_add (MCEVENT_GROUP_CORE, "vfs_timestamp", check_other_panel_timestamp, NULL, NULL);
     mc_event_add (MCEVENT_GROUP_CORE, "vfs_timestamp", check_current_panel_timestamp, NULL, NULL);
 #endif /* ENABLE_VFS */
@@ -665,15 +655,20 @@ static void
 put_current_path (void)
 {
     char *cwd_path;
+    vfs_path_t *cwd_vpath;
+
     if (!command_prompt)
         return;
 
-    cwd_path = remove_encoding_from_path (current_panel->cwd);
+    cwd_vpath = remove_encoding_from_path (current_panel->cwd_vpath);
+    cwd_path = vfs_path_to_str (cwd_vpath);
     command_insert (cmdline, cwd_path, FALSE);
 
     if (cwd_path[strlen (cwd_path) - 1] != PATH_SEP)
         command_insert (cmdline, PATH_SEP_STR, FALSE);
+
     g_free (cwd_path);
+    vfs_path_free (cwd_vpath);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -682,6 +677,7 @@ static void
 put_other_path (void)
 {
     char *cwd_path;
+    vfs_path_t *cwd_vpath;
 
     if (get_other_type () != view_listing)
         return;
@@ -689,12 +685,15 @@ put_other_path (void)
     if (!command_prompt)
         return;
 
-    cwd_path = remove_encoding_from_path (other_panel->cwd);
+    cwd_vpath = remove_encoding_from_path (other_panel->cwd_vpath);
+    cwd_path = vfs_path_to_str (cwd_vpath);
     command_insert (cmdline, cwd_path, FALSE);
 
     if (cwd_path[strlen (cwd_path) - 1] != PATH_SEP)
         command_insert (cmdline, PATH_SEP_STR, FALSE);
+
     g_free (cwd_path);
+    vfs_path_free (cwd_vpath);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -707,12 +706,13 @@ put_link (WPanel * panel)
     if (S_ISLNK (selection (panel)->st.st_mode))
     {
         char buffer[MC_MAXPATHLEN];
-        char *p;
+        vfs_path_t *vpath;
         int i;
 
-        p = concat_dir_and_file (panel->cwd, selection (panel)->fname);
-        i = mc_readlink (p, buffer, MC_MAXPATHLEN - 1);
-        g_free (p);
+        vpath = vfs_path_append_new (panel->cwd_vpath, selection (panel)->fname, NULL);
+        i = mc_readlink (vpath, buffer, MC_MAXPATHLEN - 1);
+        vfs_path_free (vpath);
+
         if (i > 0)
         {
             buffer[i] = '\0';
@@ -751,12 +751,14 @@ put_prog_name (void)
     if (get_current_type () == view_tree)
     {
         WTree *tree = (WTree *) get_panel_widget (get_current_index ());
-        tmp = tree_selected_name (tree);
+
+        tmp = vfs_path_to_str (tree_selected_name (tree));
     }
     else
-        tmp = selection (current_panel)->fname;
+        tmp = g_strdup (selection (current_panel)->fname);
 
     command_insert (cmdline, tmp, TRUE);
+    g_free (tmp);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -844,12 +846,16 @@ setup_mc (void)
 static void
 setup_dummy_mc (void)
 {
-    char d[MC_MAXPATHLEN];
+    vfs_path_t *vpath;
+    char *d;
     int ret;
 
-    mc_get_current_wd (d, MC_MAXPATHLEN);
+    d = _vfs_get_cwd ();
     setup_mc ();
-    ret = mc_chdir (d);
+    vpath = vfs_path_from_str (d);
+    ret = mc_chdir (vpath);
+    vfs_path_free (vpath);
+    g_free (d);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -871,10 +877,22 @@ done_mc (void)
     g_free (curr_dir);
 
     if ((current_panel != NULL) && (get_current_type () == view_listing))
-        vfs_stamp_path (current_panel->cwd);
+    {
+        char *tmp_path;
+
+        tmp_path = vfs_path_to_str (current_panel->cwd_vpath);
+        vfs_stamp_path (tmp_path);
+        g_free (tmp_path);
+    }
 
     if ((other_panel != NULL) && (get_other_type () == view_listing))
-        vfs_stamp_path (other_panel->cwd);
+    {
+        char *tmp_path;
+
+        tmp_path = vfs_path_to_str (other_panel->cwd_vpath);
+        vfs_stamp_path (tmp_path);
+        g_free (tmp_path);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -907,28 +925,18 @@ create_panels_and_run_mc (void)
 /* --------------------------------------------------------------------------------------------- */
 
 /** result must be free'd (I think this should go in util.c) */
-static char *
+static vfs_path_t *
 prepend_cwd_on_local (const char *filename)
 {
-    char *d;
-    size_t l;
     vfs_path_t *vpath;
 
     vpath = vfs_path_from_str (filename);
     if (!vfs_file_is_local (vpath) || g_path_is_absolute (filename))
-    {
-        vfs_path_free (vpath);
-        return g_strdup (filename);
-    }
+        return vpath;
+
     vfs_path_free (vpath);
 
-    d = g_malloc (MC_MAXPATHLEN + strlen (filename) + 2);
-    mc_get_current_wd (d, MC_MAXPATHLEN);
-    l = strlen (d);
-    d[l++] = PATH_SEP;
-    strcpy (d + l, filename);
-    canonicalize_pathname (d);
-    return d;
+    return vfs_path_append_new (vfs_get_raw_current_dir (), filename, NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -936,32 +944,48 @@ prepend_cwd_on_local (const char *filename)
 /** Invoke the internal view/edit routine with:
  * the default processing and forcing the internal viewer/editor
  */
-static void
+static gboolean
 mc_maybe_editor_or_viewer (void)
 {
+    int ret;
+
     switch (mc_global.mc_run_mode)
     {
 #ifdef USE_INTERNAL_EDIT
     case MC_RUN_EDITOR:
-        edit_file (mc_run_param0, mc_args__edit_start_line);
+        {
+            vfs_path_t *vpath = NULL;
+
+            if (mc_run_param0 != NULL && *mc_run_param0 != '\0')
+                vpath = vfs_path_from_str (mc_run_param0);
+
+            ret = edit_file (vpath, mc_args__edit_start_line);
+            vfs_path_free (vpath);
+        }
         break;
 #endif /* USE_INTERNAL_EDIT */
     case MC_RUN_VIEWER:
         {
-            char *path;
-            path = prepend_cwd_on_local (mc_run_param0);
-            view_file (path, 0, 1);
-            g_free (path);
+            vfs_path_t *vpath = NULL;
+
+            if (mc_run_param0 != NULL && *mc_run_param0 != '\0')
+                vpath = prepend_cwd_on_local (mc_run_param0);
+
+            view_file (vpath, 0, 1);
+            vfs_path_free (vpath);
+            ret = 1;
             break;
         }
 #ifdef USE_DIFF_VIEW
     case MC_RUN_DIFFVIEWER:
-        diff_view (mc_run_param0, mc_run_param1, mc_run_param0, mc_run_param1);
+        ret = dview_diff_cmd (mc_run_param0, mc_run_param1);
         break;
 #endif /* USE_DIFF_VIEW */
     default:
-        break;
+        ret = 0;
     }
+
+    return (ret != 0);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1184,7 +1208,7 @@ midnight_execute_cmd (Widget * sender, unsigned long command)
         else
             info_cmd_no_menu ();        /* shortcut or buttonbar */
         break;
-#ifdef WITH_BACKGROUND
+#ifdef ENABLE_BACKGROUND
     case CK_Jobs:
         jobs_cmd ();
         break;
@@ -1294,6 +1318,15 @@ midnight_execute_cmd (Widget * sender, unsigned long command)
     case CK_SplitVertHoriz:
         toggle_panels_split ();
         break;
+    case CK_SplitEqual:
+        panels_split_equal ();
+        break;
+    case CK_SplitMore:
+        panels_split_more ();
+        break;
+    case CK_SplitLess:
+        panels_split_less ();
+        break;
     case CK_PanelTree:
         panel_tree_cmd ();
         break;
@@ -1478,25 +1511,26 @@ midnight_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void
         return MSG_HANDLED;
 
     case DLG_UNHANDLED_KEY:
-        if (command_prompt)
         {
-            cb_ret_t v;
+            cb_ret_t v = MSG_NOT_HANDLED;
 
-            v = send_message ((Widget *) cmdline, WIDGET_KEY, parm);
-            if (v == MSG_HANDLED)
-                return MSG_HANDLED;
+            if (ctl_x_map_enabled)
+            {
+                ctl_x_map_enabled = FALSE;
+                command = keybind_lookup_keymap_command (main_x_map, parm);
+            }
+            else
+                command = keybind_lookup_keymap_command (main_map, parm);
+
+            if (command != CK_IgnoreKey)
+                v = midnight_execute_cmd (NULL, command);
+
+            if (v == MSG_NOT_HANDLED && command_prompt)
+                v = send_message ((Widget *) cmdline, WIDGET_KEY, parm);
+
+            return v;
         }
 
-        if (ctl_x_map_enabled)
-        {
-            ctl_x_map_enabled = FALSE;
-            command = keybind_lookup_keymap_command (main_x_map, parm);
-        }
-        else
-            command = keybind_lookup_keymap_command (main_map, parm);
-
-        return (command == CK_IgnoreKey) ? MSG_NOT_HANDLED : midnight_execute_cmd (NULL, command);
-
     case DLG_POST_KEY:
         if (!the_menubar->is_active)
             update_dirty_panels ();
@@ -1534,8 +1568,8 @@ midnight_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void
 void
 update_menu (void)
 {
-    menu_set_name (left_menu, horizontal_split ? _("&Above") : _("&Left"));
-    menu_set_name (right_menu, horizontal_split ? _("&Below") : _("&Right"));
+    menu_set_name (left_menu, panels_layout.horizontal_split ? _("&Above") : _("&Left"));
+    menu_set_name (right_menu, panels_layout.horizontal_split ? _("&Below") : _("&Right"));
     menubar_arrange (the_menubar);
     menubar_set_visible (the_menubar, menubar_visible);
 }
@@ -1611,9 +1645,9 @@ save_cwds_stat (void)
 {
     if (panels_options.fast_reload)
     {
-        mc_stat (current_panel->cwd, &(current_panel->dir_stat));
+        mc_stat (current_panel->cwd_vpath, &(current_panel->dir_stat));
         if (get_other_type () == view_listing)
-            mc_stat (other_panel->cwd, &(other_panel->dir_stat));
+            mc_stat (other_panel->cwd_vpath, &(other_panel->dir_stat));
     }
 }
 
@@ -1629,9 +1663,11 @@ quiet_quit_cmd (void)
 /* --------------------------------------------------------------------------------------------- */
 
 /** Run the main dialog that occupies the whole screen */
-void
+gboolean
 do_nc (void)
 {
+    gboolean ret;
+
     dlg_colors_t midnight_colors;
 
     midnight_colors[DLG_COLOR_NORMAL] = mc_skin_color_get ("dialog", "_default_");
@@ -1654,14 +1690,15 @@ do_nc (void)
 
     /* Check if we were invoked as an editor or file viewer */
     if (mc_global.mc_run_mode != MC_RUN_FULL)
-        mc_maybe_editor_or_viewer ();
+        ret = mc_maybe_editor_or_viewer ();
     else
     {
         create_panels_and_run_mc ();
+        ret = TRUE;
 
-        /* destroy_dlg destroys even current_panel->cwd, so we have to save a copy :) */
+        /* destroy_dlg destroys even current_panel->cwd_vpath, so we have to save a copy :) */
         if (mc_args__last_wd_file != NULL && vfs_current_is_local ())
-            last_wd_string = g_strdup (current_panel->cwd);
+            last_wd_string = vfs_path_to_str (current_panel->cwd_vpath);
 
         /* don't handle VFS timestamps for dirs opened in panels */
         mc_event_destroy (MCEVENT_GROUP_CORE, "vfs_timestamp");
@@ -1670,7 +1707,7 @@ do_nc (void)
     }
 
     /* Program end */
-    mc_global.widget.midnight_shutdown = TRUE;
+    mc_global.midnight_shutdown = TRUE;
     dialog_switch_shutdown ();
     done_mc ();
     destroy_dlg (midnight_dlg);
@@ -1682,6 +1719,8 @@ do_nc (void)
 
     if ((quit & SUBSHELL_EXIT) == 0)
         clr_scr ();
+
+    return ret;
 }
 
 /* --------------------------------------------------------------------------------------------- */
