@@ -35,15 +35,6 @@ enum list_types
     list_user                   /* User defined */
 };
 
-typedef enum
-{
-    view_listing = 0,           /* Directory listing */
-    view_info = 1,              /* Information panel */
-    view_tree = 2,              /* Tree view */
-    view_quick = 3,             /* Quick view */
-    view_nothing = 4,           /* Undefined */
-} panel_view_mode_t;
-
 typedef enum
 {
     frame_full,                 /* full screen frame */
@@ -52,8 +43,8 @@ typedef enum
 
 typedef enum
 {
-    UP_OPTIMIZE     = 0,
-    UP_RELOAD       = 1,
+    UP_OPTIMIZE = 0,
+    UP_RELOAD = 1,
     UP_ONLY_CURRENT = 2
 } panel_update_flags_t;
 
@@ -75,11 +66,18 @@ typedef struct panel_field_struct
     sortfn *sort_routine;       /* used by mouse_sort_col() */
 } panel_field_t;
 
+typedef struct
+{
+    dir_list list;
+    int count;
+    vfs_path_t *root_vpath;
+} panelized_panel_t;
+
 typedef struct panel_sort_info_struct
 {
-    gboolean reverse;                /* Show listing in reverse? */
-    gboolean case_sensitive;         /* Listing is case sensitive? */
-    gboolean exec_first;             /* Show executable top in list? */
+    gboolean reverse;           /* Show listing in reverse? */
+    gboolean case_sensitive;    /* Listing is case sensitive? */
+    gboolean exec_first;        /* Show executable top in list? */
     const panel_field_t *sort_field;
 } panel_sort_info_t;
 
@@ -90,8 +88,8 @@ typedef struct WPanel
 
     int list_type;              /* listing type (was view_type) */
     int active;                 /* If panel is currently selected */
-    char cwd[MC_MAXPATHLEN];    /* Current Working Directory */
-    char lwd[MC_MAXPATHLEN];    /* Last Working Directory */
+    vfs_path_t *cwd_vpath;      /* Current Working Directory */
+    vfs_path_t *lwd_vpath;      /* Last Working Directory */
     GList *dir_history;         /* directory history */
     char *hist_name;            /* directory history name for history file */
     int count;                  /* Number of files in dir structure */
@@ -101,7 +99,7 @@ typedef struct WPanel
     int top_file;               /* The file showed on the top of the panel */
     int selected;               /* Index to the selected file */
     int split;                  /* Split panel to allow two columns */
-    int is_panelized;           /* Flag: special filelisting, can't reload */
+    gboolean is_panelized;      /* Flag: special filelisting, can't reload */
     panel_display_t frame_size; /* half or full frame */
     char *filter;               /* File name filter */
     panel_sort_info_t sort_info;        /* Sort descriptor */
@@ -127,14 +125,15 @@ typedef struct WPanel
     char prev_search_buffer[MC_MAXFILENAMELEN];
     char search_char[MB_LEN_MAX];       /*buffer for multibytes characters */
     int search_chpoint;         /*point after last characters in search_char */
+    int content_shift;          /* Number of characters of filename need to skip from left side. */
+    int max_shift;              /* Max shift for visible part of current panel */
 } WPanel;
 
 /*** global variables defined in .c file *********************************************************/
 
-extern panel_field_t panel_fields[];
+extern panelized_panel_t panelized_panel;
 
-extern int torben_fj_mode;
-extern int show_mini_info;
+extern panel_field_t panel_fields[];
 
 extern hook_t *select_file_hook;
 
@@ -151,6 +150,7 @@ void panel_change_encoding (WPanel * panel);
 
 void update_panels (panel_update_flags_t flags, const char *current_file);
 int set_panel_formats (WPanel * p);
+void panel_update_cols (Widget * widget, panel_display_t frame_size);
 
 void try_to_select (WPanel * panel, const char *name);
 
@@ -161,11 +161,11 @@ void recalculate_panel_summary (WPanel * panel);
 void file_mark (WPanel * panel, int idx, int val);
 void do_file_mark (WPanel * panel, int idx, int val);
 
-gboolean do_panel_cd (struct WPanel *panel, const char *new_dir, enum cd_enum cd_type);
+gboolean do_panel_cd (struct WPanel *panel, const vfs_path_t * new_dir_vpath, enum cd_enum cd_type);
 
 void directory_history_add (struct WPanel *panel, const char *dir);
 
-char *remove_encoding_from_path (const char *path);
+vfs_path_t *remove_encoding_from_path (const vfs_path_t * vpath);
 
 gsize panel_get_num_of_sortable_fields (void);
 const char **panel_get_sortable_fields (gsize *);
@@ -174,6 +174,8 @@ const panel_field_t *panel_get_field_by_title (const char *);
 const panel_field_t *panel_get_field_by_title_hotkey (const char *);
 gsize panel_get_num_of_user_possible_fields (void);
 const char **panel_get_user_possible_fields (gsize *);
+void panel_set_cwd (WPanel * panel, const char *path_str);
+void panel_set_lwd (WPanel * panel, const char *path_str);
 
 void panel_init (void);
 void panel_deinit (void);
