@@ -1,20 +1,25 @@
-/* User Menu implementation
+/*
+   User Menu implementation
+
    Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2006, 2007 Free Software Foundation, Inc.
+   2006, 2007, 2011
+   The Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** \file usermenu.c
  *  \brief Source: user menu implementation
@@ -32,7 +37,7 @@
 #include "lib/tty/tty.h"
 #include "lib/skin.h"
 #include "lib/search.h"
-#include "lib/vfs/mc-vfs/vfs.h"
+#include "lib/vfs/vfs.h"
 #include "lib/strutil.h"
 #include "lib/util.h"
 #include "lib/widget.h"
@@ -234,7 +239,13 @@ test_condition (WEdit * edit_widget, char *p, int *condition)
             p = extract_arg (p, arg, sizeof (arg));
 #ifdef USE_INTERNAL_EDIT
             if (edit_widget != NULL)
-                *condition = mc_search (arg, edit_get_file_name (edit_widget), search_type) ? 1 : 0;
+            {
+                char *edit_filename;
+
+                edit_filename = edit_get_file_name (edit_widget);
+                *condition = mc_search (arg, edit_filename, search_type) ? 1 : 0;
+                g_free (edit_filename);
+            }
             else
 #endif
                 *condition = panel != NULL &&
@@ -255,7 +266,13 @@ test_condition (WEdit * edit_widget, char *p, int *condition)
             break;
         case 'd':
             p = extract_arg (p, arg, sizeof (arg));
-            *condition = panel != NULL && mc_search (arg, panel->cwd, search_type) ? 1 : 0;
+            {
+                char *cwd_str;
+
+                cwd_str = vfs_path_to_str (panel->cwd_vpath);
+                *condition = panel != NULL && mc_search (arg, cwd_str, search_type) ? 1 : 0;
+                g_free (cwd_str);
+            }
             break;
         case 't':
             p = extract_arg (p, arg, sizeof (arg));
@@ -404,7 +421,7 @@ test_line (WEdit * edit_widget, char *p, int *result)
 /** FIXME: recode this routine on version 3.0, it could be cleaner */
 
 static void
-execute_menu_command (WEdit * edit_widget, const char *commands)
+execute_menu_command (WEdit * edit_widget, const char *commands, gboolean show_prompt)
 {
     FILE *cmd_file;
     int cmd_file_fd;
@@ -413,7 +430,7 @@ execute_menu_command (WEdit * edit_widget, const char *commands)
     gboolean do_quote = FALSE;
     char lc_prompt[80];
     int col;
-    char *file_name;
+    vfs_path_t *file_name_vpath;
     int run_view = 0;
 
     /* Skip menu entry title line */
@@ -423,12 +440,13 @@ execute_menu_command (WEdit * edit_widget, const char *commands)
         return;
     }
 
-    cmd_file_fd = mc_mkstemps (&file_name, "mcusr", SCRIPT_SUFFIX);
+    cmd_file_fd = mc_mkstemps (&file_name_vpath, "mcusr", SCRIPT_SUFFIX);
 
     if (cmd_file_fd == -1)
     {
         message (D_ERROR, MSG_ERROR, _("Cannot create temporary command file\n%s"),
                  unix_error_string (errno));
+        vfs_path_free (file_name_vpath);
         return;
     }
     cmd_file = fdopen (cmd_file_fd, "w");
@@ -461,8 +479,8 @@ execute_menu_command (WEdit * edit_widget, const char *commands)
                 {
                     /* User canceled */
                     fclose (cmd_file);
-                    unlink (file_name);
-                    g_free (file_name);
+                    mc_unlink (file_name_vpath);
+                    vfs_path_free (file_name_vpath);
                     return;
                 }
                 if (do_quote)
@@ -523,22 +541,38 @@ execute_menu_command (WEdit * edit_widget, const char *commands)
         }
     }
     fclose (cmd_file);
-    chmod (file_name, S_IRWXU);
+    mc_chmod (file_name_vpath, S_IRWXU);
     if (run_view)
     {
+        char *file_name;
+
+        file_name = vfs_path_to_str (file_name_vpath);
         mcview_viewer (file_name, NULL, 0);
+        g_free (file_name);
         dialog_switch_process_pending ();
     }
     else
     {
         /* execute the command indirectly to allow execution even
          * on no-exec filesystems. */
-        char *cmd = g_strconcat ("/bin/sh ", file_name, (char *) NULL);
-        shell_execute (cmd, EXECUTE_HIDE);
+        char *file_name, *cmd;
+
+        file_name = vfs_path_to_str (file_name_vpath);
+        cmd = g_strconcat ("/bin/sh ", file_name, (char *) NULL);
+        g_free (file_name);
+        if (!show_prompt)
+        {
+            if (system (cmd) == -1)
+                message (D_ERROR, MSG_ERROR, "%s", _("Error calling program"));
+        }
+        else
+        {
+            shell_execute (cmd, EXECUTE_HIDE);
+        }
         g_free (cmd);
     }
-    unlink (file_name);
-    g_free (file_name);
+    mc_unlink (file_name_vpath);
+    vfs_path_free (file_name_vpath);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -714,7 +748,7 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
     if (c == '%')
         return g_strdup ("%");
 
-    if (mc_run_mode == MC_RUN_FULL)
+    if (mc_global.mc_run_mode == MC_RUN_FULL)
     {
         if (g_ascii_islower ((gchar) c))
             panel = current_panel;
@@ -724,11 +758,11 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
                 return g_strdup ("");
             panel = other_panel;
         }
-        fname = panel->dir.list[panel->selected].fname;
+        fname = g_strdup (panel->dir.list[panel->selected].fname);
     }
 #ifdef USE_INTERNAL_EDIT
-    else if (mc_run_mode == MC_RUN_EDITOR)
-        fname = (char *) edit_get_file_name (edit_widget);
+    else if (mc_global.mc_run_mode == MC_RUN_EDITOR)
+        fname = edit_get_file_name (edit_widget);
 #endif
 
     if (do_quote)
@@ -742,31 +776,35 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
     {
     case 'f':
     case 'p':
-        return (*quote_func) (fname, 0);
+        result = (*quote_func) (fname, 0);
+        goto ret;
     case 'x':
-        return (*quote_func) (extension (fname), 0);
+        result = (*quote_func) (extension (fname), 0);
+        goto ret;
     case 'd':
         {
             char *cwd;
             char *qstr;
 
-            cwd = g_malloc (MC_MAXPATHLEN + 1);
-
             if (panel)
-                g_strlcpy (cwd, panel->cwd, MC_MAXPATHLEN + 1);
+                cwd = vfs_path_to_str (panel->cwd_vpath);
             else
-                mc_get_current_wd (cwd, MC_MAXPATHLEN + 1);
+                cwd = vfs_get_current_dir ();
 
             qstr = (*quote_func) (cwd, 0);
 
             g_free (cwd);
 
-            return qstr;
+            result = qstr;
+            goto ret;
         }
     case 'i':                  /* indent equal number cursor position in line */
 #ifdef USE_INTERNAL_EDIT
         if (edit_widget)
-            return g_strnfill (edit_get_curs_col (edit_widget), ' ');
+        {
+            result = g_strnfill (edit_get_curs_col (edit_widget), ' ');
+            goto ret;
+        }
 #endif
         break;
     case 'y':                  /* syntax type */
@@ -775,7 +813,10 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
         {
             const char *syntax_type = edit_get_syntax_type (edit_widget);
             if (syntax_type != NULL)
-                return g_strdup (syntax_type);
+            {
+                result = g_strdup (syntax_type);
+                goto ret;
+            }
         }
 #endif
         break;
@@ -785,62 +826,81 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
 #ifdef USE_INTERNAL_EDIT
             if (edit_widget)
             {
-                char *file = concat_dir_and_file (home_dir, EDIT_BLOCK_FILE);
-                fname = (*quote_func) (file, 0);
+                char *file;
+
+                file = mc_config_get_full_path (EDIT_BLOCK_FILE);
+                result = (*quote_func) (file, 0);
                 g_free (file);
-                return fname;
+                goto ret;
             }
 #endif
             if (c_lc == 'b')
-                return strip_ext ((*quote_func) (fname, 0));
+            {
+                result = strip_ext ((*quote_func) (fname, 0));
+                goto ret;
+            }
             break;
         }
     case 'n':                  /* strip extension in editor */
 #ifdef USE_INTERNAL_EDIT
         if (edit_widget)
-            return strip_ext ((*quote_func) (fname, 0));
+        {
+            result = strip_ext ((*quote_func) (fname, 0));
+            goto ret;
+        }
 #endif
         break;
     case 'm':                  /* menu file name */
         if (menu)
-            return (*quote_func) (menu, 0);
+        {
+            result = (*quote_func) (menu, 0);
+            goto ret;
+        }
         break;
     case 's':
         if (!panel || !panel->marked)
-            return (*quote_func) (fname, 0);
+        {
+            result = (*quote_func) (fname, 0);
+            goto ret;
+        }
 
         /* Fall through */
 
     case 't':
     case 'u':
         {
-            int length = 2, i;
-            char *block, *tmp;
+            GString *block;
+            int i;
 
-            if (!panel)
-                return g_strdup ("");
+            if (panel == NULL)
+            {
+                result = g_strdup ("");
+                goto ret;
+            }
 
-            for (i = 0; i < panel->count; i++)
-                if (panel->dir.list[i].f.marked)
-                    length += strlen (panel->dir.list[i].fname) + 1;    /* for space */
+            block = g_string_sized_new (16);
 
-            block = g_malloc (length * 2 + 1);
-            *block = 0;
             for (i = 0; i < panel->count; i++)
                 if (panel->dir.list[i].f.marked)
                 {
+                    char *tmp;
+
                     tmp = (*quote_func) (panel->dir.list[i].fname, 0);
-                    strcat (block, tmp);
+                    g_string_append (block, tmp);
+                    g_string_append_c (block, ' ');
                     g_free (tmp);
-                    strcat (block, " ");
+
                     if (c_lc == 'u')
                         do_file_mark (panel, i, 0);
                 }
-            return block;
+            result = g_string_free (block, FALSE);
+            goto ret;
         }                       /* sub case block */
     }                           /* switch */
     result = g_strdup ("% ");
     result[1] = c;
+  ret:
+    g_free (fname);
     return result;
 }
 
@@ -850,8 +910,8 @@ expand_format (struct WEdit *edit_widget, char c, gboolean do_quote)
  * otherwise we are called from the mcedit menu.
  */
 
-void
-user_menu_cmd (struct WEdit *edit_widget)
+gboolean
+user_menu_cmd (struct WEdit * edit_widget, const char *menu_file, int selected_entry)
 {
     char *p;
     char *data, **entries;
@@ -859,32 +919,55 @@ user_menu_cmd (struct WEdit *edit_widget)
     int col, i, accept_entry = 1;
     int selected, old_patterns;
     Listbox *listbox;
+    gboolean res = FALSE;
+    gboolean interactive = TRUE;
 
     if (!vfs_current_is_local ())
     {
         message (D_ERROR, MSG_ERROR, "%s", _("Cannot execute commands on non-local filesystems"));
-        return;
+        return FALSE;
     }
-
-    menu = g_strdup (edit_widget ? EDIT_LOCAL_MENU : MC_LOCAL_MENU);
+    if (menu_file != NULL)
+        menu = g_strdup (menu_file);
+    else
+        menu = g_strdup (edit_widget ? EDIT_LOCAL_MENU : MC_LOCAL_MENU);
     if (!exist_file (menu) || !menu_file_own (menu))
     {
+        if (menu_file != NULL)
+        {
+            message (D_ERROR, MSG_ERROR, _("Cannot open file %s\n%s"), menu,
+                     unix_error_string (errno));
+            g_free (menu);
+            menu = NULL;
+            return FALSE;
+        }
+
         g_free (menu);
         if (edit_widget)
-            menu = concat_dir_and_file (home_dir, EDIT_HOME_MENU);
+            menu = mc_config_get_full_path (EDIT_HOME_MENU);
         else
-            menu = g_build_filename (home_dir, MC_USERCONF_DIR, MC_USERMENU_FILE, NULL);
+            menu = mc_config_get_full_path (MC_USERMENU_FILE);
 
 
         if (!exist_file (menu))
         {
             g_free (menu);
-            menu = concat_dir_and_file (mc_home, edit_widget ? EDIT_GLOBAL_MENU : MC_GLOBAL_MENU);
+            menu =
+                mc_build_filename (mc_config_get_home_dir (),
+                                   edit_widget ? EDIT_GLOBAL_MENU : MC_GLOBAL_MENU, NULL);
             if (!exist_file (menu))
             {
                 g_free (menu);
-                menu = concat_dir_and_file
-                    (mc_home_alt, edit_widget ? EDIT_GLOBAL_MENU : MC_GLOBAL_MENU);
+                menu =
+                    mc_build_filename (mc_global.sysconfig_dir,
+                                       edit_widget ? EDIT_GLOBAL_MENU : MC_GLOBAL_MENU, NULL);
+                if (!exist_file (menu))
+                {
+                    g_free (menu);
+                    menu = mc_build_filename
+                        (mc_global.share_data_dir, edit_widget ? EDIT_GLOBAL_MENU : MC_GLOBAL_MENU,
+                         NULL);
+                }
             }
         }
     }
@@ -894,7 +977,7 @@ user_menu_cmd (struct WEdit *edit_widget)
         message (D_ERROR, MSG_ERROR, _("Cannot open file%s\n%s"), menu, unix_error_string (errno));
         g_free (menu);
         menu = NULL;
-        return;
+        return FALSE;
     }
 
     max_cols = 0;
@@ -926,6 +1009,9 @@ user_menu_cmd (struct WEdit *edit_widget)
         {
             if (*p == '#')
             {
+                /* show prompt if first line of external script is #interactive */
+                if (selected_entry >= 0 && strncmp (p, "#silent", 7) == 0)
+                    interactive = FALSE;
                 /* A commented menu entry */
                 accept_entry = 1;
             }
@@ -990,27 +1076,38 @@ user_menu_cmd (struct WEdit *edit_widget)
     }
 
     if (menu_lines == 0)
+    {
         message (D_ERROR, MSG_ERROR, _("No suitable entries found in %s"), menu);
+        res = FALSE;
+    }
     else
     {
-        max_cols = min (max (max_cols, col), MAX_ENTRY_LEN);
-
-        /* Create listbox */
-        listbox = create_listbox_window (menu_lines, max_cols + 2, _("User menu"),
-                                         "[Menu File Edit]");
-        /* insert all the items found */
-        for (i = 0; i < menu_lines; i++)
+        if (selected_entry >= 0)
+            selected = selected_entry;
+        else
         {
-            p = entries[i];
-            LISTBOX_APPEND_TEXT (listbox, (unsigned char) p[0],
-                                 extract_line (p, p + MAX_ENTRY_LEN), p);
-        }
-        /* Select the default entry */
-        listbox_select_entry (listbox->list, selected);
+            max_cols = min (max (max_cols, col), MAX_ENTRY_LEN);
 
-        selected = run_listbox (listbox);
+            /* Create listbox */
+            listbox = create_listbox_window (menu_lines, max_cols + 2, _("User menu"),
+                                             "[Menu File Edit]");
+            /* insert all the items found */
+            for (i = 0; i < menu_lines; i++)
+            {
+                p = entries[i];
+                LISTBOX_APPEND_TEXT (listbox, (unsigned char) p[0],
+                                     extract_line (p, p + MAX_ENTRY_LEN), p);
+            }
+            /* Select the default entry */
+            listbox_select_entry (listbox->list, selected);
+
+            selected = run_listbox (listbox);
+        }
         if (selected >= 0)
-            execute_menu_command (edit_widget, entries[selected]);
+        {
+            execute_menu_command (edit_widget, entries[selected], interactive);
+            res = TRUE;
+        }
 
         do_refresh ();
     }
@@ -1020,6 +1117,7 @@ user_menu_cmd (struct WEdit *edit_widget)
     menu = NULL;
     g_free (entries);
     g_free (data);
+    return res;
 }
 
 /* --------------------------------------------------------------------------------------------- */
