@@ -116,6 +116,10 @@ static const global_keymap_ini_t default_main_keymap[] = {
     {"PanelListingSwitch", "alt-t"},
     /* Swap panels */
     {"Swap", "ctrl-u"},
+    /* Resize panels */
+    {"SplitEqual", "alt-equal"},
+    {"SplitMore", "alt-shift-right"},
+    {"SplitLess", "alt-shift-left"},
     /* View output */
     {"Shell", "ctrl-o"},
     {"ShowHidden", "alt-dot"},
@@ -153,9 +157,9 @@ static const global_keymap_ini_t default_main_x_keymap[] = {
     {"PanelInfo", "i"},
     {"PanelQuickView", "q"},
     {"HotListAdd", "h"},
-#ifdef WITH_BACKGROUND
+#ifdef ENABLE_BACKGROUND
     {"Jobs", "j"},
-#endif /* WITH_BACKGROUND */
+#endif /* ENABLE_BACKGROUND */
     {"ExternalPanelize", "!"},
     {NULL, NULL}
 };
@@ -429,8 +433,8 @@ static const global_keymap_ini_t default_viewer_keymap[] = {
     {"Quit", "f3; f10; q; esc"},
     {"HexMode", "f4"},
     {"Goto", "f5"},
-    {"Search", "f7; question; slash"},
-    {"SearchContinue", "ctrl-r; ctrl-s; f17; n"},
+    {"Search", "f7"},
+    {"SearchContinue", "f17; n"},
     {"MagicMode", "f8"},
     {"NroffMode", "f9"},
     {"Home", "ctrl-a"},
@@ -456,6 +460,10 @@ static const global_keymap_ini_t default_viewer_keymap[] = {
 #endif
     {"Shell", "ctrl-o"},
     {"Ruler", "alt-r"},
+    {"SearchForward", "slash"},
+    {"SearchBackward", "question"},
+    {"SearchForwardContinue", "ctrl-s"},
+    {"SearchBackwardContinue", "ctrl-r"},
     {NULL, NULL}
 };
 
@@ -467,8 +475,8 @@ static const global_keymap_ini_t default_viewer_hex_keymap[] = {
     {"HexMode", "f4"},
     {"Goto", "f5"},
     {"Save", "f6"},
-    {"Search", "f7; question; slash"},
-    {"SearchContinue", "ctrl-r; ctrl-s; f17; n"},
+    {"Search", "f7"},
+    {"SearchContinue", "f17; n"},
     {"MagicMode", "f8"},
     {"NroffMode", "f9"},
     {"ToggleNavigation", "tab"},
@@ -486,6 +494,10 @@ static const global_keymap_ini_t default_viewer_hex_keymap[] = {
     {"SelectCodepage", "alt-e"},
 #endif
     {"Shell", "ctrl-o"},
+    {"SearchForward", "slash"},
+    {"SearchBackward", "question"},
+    {"SearchForwardContinue", "ctrl-s"},
+    {"SearchBackwardContinue", "ctrl-r"},
     {NULL, NULL}
 };
 
