@@ -217,8 +217,6 @@ static const struct
     const char *opt_name;
     int *opt_addr;
 } layout [] = {
-    { "equal_split", &equal_split },
-    { "first_panel_size", &first_panel_size },
     { "message_visible", &mc_global.message_visible },
     { "keybar_visible", &mc_global.keybar_visible },
     { "xterm_title", &xterm_title },
@@ -226,6 +224,11 @@ static const struct
     { "command_prompt", &command_prompt },
     { "menubar_visible", &menubar_visible },
     { "free_space", &free_space },
+    { "horizontal_split", &panels_layout.horizontal_split },
+    { "vertical_equal", &panels_layout.vertical_equal },
+    { "left_panel_size", &panels_layout.left_panel_size },
+    { "horizontal_equal", &panels_layout.horizontal_equal },
+    { "top_panel_size", &panels_layout.top_panel_size },
     { NULL, NULL }
 };
 
@@ -317,7 +320,6 @@ static const struct
     { "editor_group_undo", &option_group_undo },
 #endif /* USE_INTERNAL_EDIT */
     { "nice_rotating_dash", &nice_rotating_dash },
-    { "horizontal_split",   &horizontal_split },
     { "mcview_remember_file_position", &mcview_remember_file_position },
     { "auto_fill_mkdir_name", &auto_fill_mkdir_name },
     { "copymove_persistent_attr", &setup_copymove_persistent_attr },
@@ -534,10 +536,34 @@ static void
 load_layout (void)
 {
     size_t i;
+    int equal_split;
+    int first_panel_size;
 
+    /* legacy options */
+    panels_layout.horizontal_split = mc_config_get_int (mc_main_config, CONFIG_APP_SECTION,
+                                                        "horizontal_split", 0);
+    equal_split = mc_config_get_int (mc_main_config, "Layout", "equal_split", 1);
+    first_panel_size = mc_config_get_int (mc_main_config, "Layout", "first_panel_size", 1);
+    if (panels_layout.horizontal_split)
+    {
+        panels_layout.horizontal_equal = equal_split;
+        panels_layout.left_panel_size = first_panel_size;
+    }
+    else
+    {
+        panels_layout.vertical_equal = equal_split;
+        panels_layout.top_panel_size = first_panel_size;
+    }
+
+    /* actual options override legacy ones */
     for (i = 0; layout[i].opt_name != NULL; i++)
-        *layout[i].opt_addr = mc_config_get_int (mc_main_config, "Layout",
+        *layout[i].opt_addr = mc_config_get_int (mc_main_config, CONFIG_LAYOUT_SECTION,
                                                  layout[i].opt_name, *layout[i].opt_addr);
+
+    /* remove legacy options */
+    mc_config_del_key (mc_main_config, CONFIG_APP_SECTION, "horizontal_split");
+    mc_config_del_key (mc_main_config, "Layout", "equal_split");
+    mc_config_del_key (mc_main_config, "Layout", "first_panel_size");
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -764,7 +790,13 @@ save_panel_types (void)
     if (type == view_listing)
         panel_save_setup (right_panel, right_panel->panel_name);
 
-    mc_config_set_string (mc_panels_config, "Dirs", "other_dir", get_panel_dir_for (other_panel));
+    {
+        char *dirs;
+
+        dirs = get_panel_dir_for (other_panel);
+        mc_config_set_string (mc_panels_config, "Dirs", "other_dir", dirs);
+        g_free (dirs);
+    }
 
     if (current_panel != NULL)
         mc_config_set_string (mc_panels_config, "Dirs", "current_is_left",
@@ -795,7 +827,7 @@ setup_init (void)
     profile = mc_config_get_full_path (MC_CONFIG_FILE);
     if (!exist_file (profile))
     {
-        inifile = concat_dir_and_file (mc_global.sysconfig_dir, "mc.ini");
+        inifile = mc_build_filename (mc_global.sysconfig_dir, "mc.ini", NULL);
         if (exist_file (inifile))
         {
             g_free (profile);
@@ -804,7 +836,7 @@ setup_init (void)
         else
         {
             g_free (inifile);
-            inifile = concat_dir_and_file (mc_global.share_data_dir, "mc.ini");
+            inifile = mc_build_filename (mc_global.share_data_dir, "mc.ini", NULL);
             if (exist_file (inifile))
             {
                 g_free (profile);
@@ -908,13 +940,15 @@ load_setup (void)
 
     /* Load time formats */
     user_recent_timeformat =
-        mc_config_get_string (mc_main_config, "Misc", "timeformat_recent", FMTTIME);
-    user_old_timeformat = mc_config_get_string (mc_main_config, "Misc", "timeformat_old", FMTYEAR);
+        mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "timeformat_recent", FMTTIME);
+    user_old_timeformat =
+        mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "timeformat_old", FMTYEAR);
 
 #ifdef ENABLE_VFS_FTP
-    ftpfs_proxy_host = mc_config_get_string (mc_main_config, "Misc", "ftp_proxy_host", "gate");
-    ftpfs_ignore_chattr_errors = mc_config_get_bool (mc_main_config, CONFIG_APP_SECTION,
-                                                     "ignore_ftp_chattr_errors", TRUE);
+    ftpfs_proxy_host =
+        mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "ftp_proxy_host", "gate");
+    ftpfs_ignore_chattr_errors =
+        mc_config_get_bool (mc_main_config, CONFIG_APP_SECTION, "ignore_ftp_chattr_errors", TRUE);
     ftpfs_init_passwd ();
 #endif /* ENABLE_VFS_FTP */
 
@@ -933,14 +967,14 @@ load_setup (void)
 #ifdef HAVE_CHARSET
     if (codepages->len > 1)
     {
-        buffer = mc_config_get_string (mc_main_config, "Misc", "display_codepage", "");
+        buffer = mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "display_codepage", "");
         if (buffer[0] != '\0')
         {
             mc_global.display_codepage = get_codepage_index (buffer);
             cp_display = get_codepage_id (mc_global.display_codepage);
         }
         g_free (buffer);
-        buffer = mc_config_get_string (mc_main_config, "Misc", "source_codepage", "");
+        buffer = mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "source_codepage", "");
         if (buffer[0] != '\0')
         {
             default_source_codepage = get_codepage_index (buffer);
@@ -950,7 +984,8 @@ load_setup (void)
         g_free (buffer);
     }
 
-    autodetect_codeset = mc_config_get_string (mc_main_config, "Misc", "autodetect_codeset", "");
+    autodetect_codeset =
+        mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "autodetect_codeset", "");
     if ((autodetect_codeset[0] != '\0') && (strcmp (autodetect_codeset, "off") != 0))
         is_autodetect_codeset_enabled = TRUE;
 
@@ -960,8 +995,10 @@ load_setup (void)
         mc_global.utf8_display = str_isutf8 (buffer);
 #endif /* HAVE_CHARSET */
 
-    clipboard_store_path = mc_config_get_string (mc_main_config, "Misc", "clipboard_store", "");
-    clipboard_paste_path = mc_config_get_string (mc_main_config, "Misc", "clipboard_paste", "");
+    clipboard_store_path =
+        mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "clipboard_store", "");
+    clipboard_paste_path =
+        mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "clipboard_paste", "");
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -989,20 +1026,25 @@ save_setup (gboolean save_options, gboolean save_panel_options)
         /* directory_history_save (); */
 
 #ifdef ENABLE_VFS_FTP
-        mc_config_set_string (mc_main_config, "Misc", "ftpfs_password", ftpfs_anonymous_passwd);
+        mc_config_set_string (mc_main_config, CONFIG_MISC_SECTION, "ftpfs_password",
+                              ftpfs_anonymous_passwd);
         if (ftpfs_proxy_host)
-            mc_config_set_string (mc_main_config, "Misc", "ftp_proxy_host", ftpfs_proxy_host);
+            mc_config_set_string (mc_main_config, CONFIG_MISC_SECTION, "ftp_proxy_host",
+                                  ftpfs_proxy_host);
 #endif /* ENABLE_VFS_FTP */
 
 #ifdef HAVE_CHARSET
-        mc_config_set_string (mc_main_config, "Misc", "display_codepage",
+        mc_config_set_string (mc_main_config, CONFIG_MISC_SECTION, "display_codepage",
                               get_codepage_id (mc_global.display_codepage));
-        mc_config_set_string (mc_main_config, "Misc", "source_codepage",
+        mc_config_set_string (mc_main_config, CONFIG_MISC_SECTION, "source_codepage",
                               get_codepage_id (default_source_codepage));
-        mc_config_set_string (mc_main_config, "Misc", "autodetect_codeset", autodetect_codeset);
+        mc_config_set_string (mc_main_config, CONFIG_MISC_SECTION, "autodetect_codeset",
+                              autodetect_codeset);
 #endif /* HAVE_CHARSET */
-        mc_config_set_string (mc_main_config, "Misc", "clipboard_store", clipboard_store_path);
-        mc_config_set_string (mc_main_config, "Misc", "clipboard_paste", clipboard_paste_path);
+        mc_config_set_string (mc_main_config, CONFIG_MISC_SECTION, "clipboard_store",
+                              clipboard_store_path);
+        mc_config_set_string (mc_main_config, CONFIG_MISC_SECTION, "clipboard_paste",
+                              clipboard_paste_path);
 
         tmp_profile = mc_config_get_full_path (MC_CONFIG_FILE);
         ret = mc_config_save_to_file (mc_main_config, tmp_profile, NULL);
@@ -1088,7 +1130,8 @@ save_layout (void)
 
     /* Save integer options */
     for (i = 0; layout[i].opt_name != NULL; i++)
-        mc_config_set_int (mc_main_config, "Layout", layout[i].opt_name, *layout[i].opt_addr);
+        mc_config_set_int (mc_main_config, CONFIG_LAYOUT_SECTION, layout[i].opt_name,
+                           *layout[i].opt_addr);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1122,7 +1165,7 @@ load_anon_passwd (void)
 {
     char *buffer;
 
-    buffer = mc_config_get_string (mc_main_config, "Misc", "ftpfs_password", "");
+    buffer = mc_config_get_string (mc_main_config, CONFIG_MISC_SECTION, "ftpfs_password", "");
 
     if ((buffer != NULL) && (buffer[0] != '\0'))
         return buffer;
