@@ -53,12 +53,12 @@ teardown (void)
 }
 
 static int
-test_chdir(const vfs_path_t * vpath)
+test_chdir (const vfs_path_t * vpath)
 {
 #if 0
-    char *path = vfs_path_to_str(vpath);
-    printf("test_chdir: %s\n", path);
-    g_free(path);
+    char *path = vfs_path_to_str (vpath);
+    printf ("test_chdir: %s\n", path);
+    g_free (path);
 #else
     (void) vpath;
 #endif
@@ -68,16 +68,21 @@ test_chdir(const vfs_path_t * vpath)
 /* --------------------------------------------------------------------------------------------- */
 
 #define cd_and_check( cd_dir, etalon ) \
-    mc_chdir(cd_dir); \
+    vpath = vfs_path_from_str (cd_dir); \
+    mc_chdir(vpath); \
+    vfs_path_free (vpath); \
+    buffer = _vfs_get_cwd (); \
     fail_unless( \
-        strcmp(etalon, mc_get_current_wd(buffer,MC_MAXPATHLEN)) == 0, \
-        "\n expected(%s) doesn't equal \nto actual(%s)", etalon, buffer);
+        strcmp(etalon, buffer) == 0, \
+        "\n expected(%s) doesn't equal \nto actual(%s)", etalon, buffer); \
+    g_free (buffer);
 
 START_TEST (set_up_current_dir_url)
 {
+    vfs_path_t *vpath;
     static struct vfs_s_subclass test_subclass;
     static struct vfs_class vfs_test_ops;
-    char buffer[MC_MAXPATHLEN];
+    char *buffer;
 
     vfs_s_init_class (&vfs_test_ops, &test_subclass);
 
@@ -103,11 +108,12 @@ START_TEST (set_up_current_dir_url)
     test_subclass.flags = VFS_S_REMOTE;
 
     cd_and_check ("/test://user:pass@host.net/path", "/test://user:pass@host.net/path");
-    cd_and_check ("..", "/test://user:pass@host.net");
+    cd_and_check ("..", "/test://user:pass@host.net/");
 
     cd_and_check ("..", "/");
 
 }
+
 END_TEST
 
 /* --------------------------------------------------------------------------------------------- */
